import argparse, os, shutil, traceback, sys
import pyplume.tests.testMechs

def mechFileAdd(fname):
    """
    This function takes a given mechanism file and stores it for use with the
    plume model generator.

    This will automatically replace mechanism files in the program folder. They can be restored however with
    mechFileRestore()
    """
    #get directory path of mechanisms
    direc = os.path.join(os.path.dirname(os.path.abspath(__file__)),'mechanisms')
    #check if file exists - throw error if not
    print("\nChecking {} exists and is a file...\n".format(os.path.basename(fname)))
    try:
        if not os.path.exists(fname) or not os.path.isfile(fname):
            raise TypeError("{}: does not exist or is not a file.".format(fname))
    except TypeError as e:
        tb = traceback.format_exc()
        print(tb)
        sys.exit()
    #Copy file to new directory
    print("Adding {} to mechanisms...\n".format(os.path.basename(fname)))
    shutil.copyfile(fname,os.path.join(direc,os.path.basename(fname)))
    print("done.\n")

def mechFileRestore():
    """Use this function to restore the original mechanism files supplied."""
    #get directory path of package
    print("\nRestoring original mechanism files...\n")
    direc = os.path.dirname(os.path.abspath(__file__))
    opath = os.path.join(direc,"originals")
    npath = os.path.join(direc,"mechanisms")
    if not os.path.exists(npath):
        os.mkdir(npath)
    for f in os.listdir(opath):
        print("\tRestoring: {}...".format(f))
        shutil.copyfile(os.path.join(opath,f),os.path.join(npath,f))
    print("done.\n")

def mechFileDelete(fname):
    """Use this function to delete a mechanism file supplied."""
    #get directory path of package
    print("\nRemoving {} mechanism file...\n".format(fname))
    direc = os.path.dirname(os.path.abspath(__file__))
    npath = os.path.join(direc,"mechanisms")
    os.remove(os.path.join(npath,fname))
    print('done.\n')

def mechFileList():
    """Use this function to list mechanism files supplied."""
    #get directory path of package
    print("\nListing mechanism files.\n")
    direc = os.path.dirname(os.path.abspath(__file__))
    npath = os.path.join(direc,"mechanisms")
    for file in os.listdir(npath):
        print("\t"+file)
    print("done.\n")

def mechCLI():
    """This function is used for the command line interface option of mech"""
    parser = argparse.ArgumentParser(description="""This is the commandline interface for
    managing mechanism files of PyPlume.
    """)
    parser.add_argument("-r","--restore",action='store_true',help="""set this flag to restore mechanism files.""")
    parser.add_argument("-l","--list",action='store_true',help="""set this flag to list mechanism files.""")
    parser.add_argument('-a','--add',default=None,type=str,help="""this can be used to add a mechanism file to the codes internal data.""")
    parser.add_argument('-d','--delete',default=None,type=str,help="""this can be used to delete a mechanism file to the codes internal data.""")
    parser.add_argument("-t","--test",action='store_true',help="""set this flag to run test functions.""")
    args = parser.parse_args()

    if args.restore:
        mechFileRestore()

    if args.add is not None:
        mechFileAdd(args.add)

    if args.delete is not None:
        mechFileDelete(args.delete)

    if args.list:
        mechFileList()

    if args.test:
        pyplume.tests.testMechs.runTests()

if __name__ == "__main__":
    mechCLI()




import sys,os,pytest,filecmp,shutil,pytest
main_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(main_dir,'..'))
import mech

def getMechPath(fname=None):
    """This function gets the mechanism path. NOT A TEST"""
    newfile = os.path.join(main_dir,'..')
    pathList=['mechanisms',]
    if fname is not None:
        pathList.append(fname)
    for item in pathList:
        newfile = os.path.join(newfile,item)
    return newfile

def test_mechFileAdd_exit(fname="gri30"):
    """Test that the function exits properly with bad input"""
    with pytest.raises(SystemExit) as output:
        mech.mechFileAdd(fname)
        assert output.type == SystemExit
        assert output.value.code == 42

def test_mechFileAdd(fname=os.path.join(main_dir,"gri30.cti")):
    """This function tests adding the mechanism files."""
    #Run file copy
    mech.mechFileAdd(fname)
    #Create new file path
    newfile = getMechPath(fname=os.path.basename(fname))
    #Compare files
    assert filecmp.cmp(fname,newfile,shallow=False)

def test_mechFileDelete(fname="Ajgkdlsageiwon.cti"):
    """This function tests adding the mechanism files."""
    #Get directory name
    fname = getMechPath(fname=os.path.basename(fname))
    #Create file
    f = open(fname,'w')
    f.close()
    #delete file
    mech.mechFileDelete(fname)
    #List files in directory
    dirFiles = os.listdir()
    #Compare files
    assert os.path.basename(fname) not in dirFiles

def test_mechFileRestore():
    """Use this function to test that the files are restored properly."""
    #Get paths
    mPath = getMechPath()
    omPath = os.path.join(os.path.join(mPath,'..'),"originals")
    #Remove folder of mechanisms
    shutil.rmtree(mPath)
    #Run restore
    mech.mechFileRestore()
    #Compare contents
    for f in os.listdir(omPath):
        ofile = os.path.join(omPath,f)
        nfile = os.path.join(mPath,f)
        assert filecmp.cmp(ofile,nfile,shallow=False)

def runTests():
    """Use this function to run pytests."""
    pytest.main([__file__,"--verbose"])



import sys,os,pytest,filecmp,shutil,pytest
main_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(main_dir,'..'))
import model
import itertools
import numpy as np

def test_linearExpansionModel():
    """This function tests that linearExpansion connects is working properly."""
    #This portions test to make sure the code fails with n values that cause non-integer steps
    with pytest.raises(SystemExit) as output:
        model.PlumeModel.linearExpansionModel(14)
        assert output.type == SystemExit
        assert output.value.code == 42

    #Manually check that it produces correct adj matrix otherwise
    n=10
    connects = model.PlumeModel.linearExpansionModel(n).connects
    ones = [(0,1),(0,2),(1,3),(1,4),(1,5),(2,3),(2,4),(2,5)]
    ones += [(3,6),(3,7),(3,8),(3,9),(4,6),(4,7),(4,8),(4,9),(5,6),(5,7),(5,8),(5,9)]
    ones += [(n,0),(n,1),(n,2),(n,3),(n,5),(n,6),(n,9)]
    trange = range(connects.shape[0])
    for i in trange:
        for j in trange:
            if (i,j) in ones:
                assert connects[i,j] == 1
            else:
                assert connects[i,j] == 0


def test_gridModel():
    """Use this function to test grid model connects matrix."""
    #Manually check that it produces correct adj matrix otherwise
    n = m = 3
    connects = model.PlumeModel.gridModel(n=n,m=m).connects
    ones = np.zeros(connects.shape)
    diag=4
    ones[0,1:diag] = 1
    diagStart = diag
    for i in range(1,int(n*m-2)):
        ones[i,diagStart] = 1
        diagStart+=1
    for ext in [0,1,3,4,6,7,9]:
        ones[-1,ext] = 1
    trange = range(connects.shape[0])
    for i in trange:
        for j in trange:
            assert connects[i,j] == ones[i,j]

def test_simpleModel():
    """Use this function to test grid model connects matrix."""
    #Manually check that it produces correct adj matrix otherwise
    n = m = 3
    connects = model.PlumeModel.simpleModel().connects
    ones = np.zeros(connects.shape)
    for i,row in enumerate(connects):
        for j,val in enumerate(row):
            if i == 1 and j == 0:
                assert connects[i,j] == 1
            else:
                assert connects[i,j] == 0

def test_connectReactors():
    """Use this function to test the mass flow connections"""

def test_createReactors():
    """Use this function to test creation of reactors."""

def runTests():
    """Use this function to run pytests."""
    pytest.main([__file__,"--verbose"])



# @Author: Anthony Walker
# @Date:   2020-04-09T12:53:51-07:00
# @Email:  walkanth@oregonstate.edu
# @Filename: gs.py
# @Last modified by:   Anthony Walker
# @Last modified time: 2020-04-10T09:53:34-07:00

#imports
import cantera as ct
import itertools as it
import numpy as np
import sys,os,traceback,output,argparse
import pyplume.tests.testModel

class PlumeModel(object):
    """PlumeModel class is used to generate a reactor network for modeling exhaust plume"""

    def __init__(self, mechs, connects, residenceTime=lambda t: 0.1, entrainment=lambda t:0.1,fpath=None,setCanteraPath=None,build=False):
        """constructor for plume model.
        Parameters:
        mechs - an array like structure with at least 3 mechanisms, [fuelMech,atmMech,eMech1,eMech2,...,eMechN]
        residenceTime - a function that specifies the residence time as a function of time---this is used to determine combustor and system mass flow rates.
        entrainment - a function that specifies entrainment mass as a function of time.
        connects - an 2d adjacency matrix with integer values corresponding to the appropriate mass flow function+1 in the list of mass flow functions.
                    So, the first mass flow function, 0 index, will be represented as 1 in the matrix. This is because these values will be used for conditionals
                    as well. A template matrix can be generated. The matrix should specifically
        setCanteraPath - path variable to cantera mech files
        build -  boolean that builds network strictly from configuration in mechanism files (T,P) if true.
            default: build=false
        """
        super(PlumeModel, self).__init__()
        #Saving contents into self of inputs
        self.mechs = mechs #mechanisms used in gas creation
        self.connects = connects #Adjacency matrix of connections
        self.residenceTime = residenceTime #RTF a function that controlls mass flow
        self.entrainment = entrainment #a function that specifies mass entrainment
        self.build = build #This is a bool that says to build on construction
        self.nex = connects.shape[0]-1 #This is the number of exhaust reactors
        self.fpath=fpath
        self.ptype=True #True for sparse printing
        # Add cantera or mechanisms path
        if setCanteraPath is not None:
            ct.add_directory(setCanteraPath)
        else:
            self.mechPath = os.path.dirname(os.path.abspath(__file__))
            self.mechPath = os.path.join(self.mechPath,"mechanisms")
            sys.path.append(self.mechPath)
        #Creation of gas objects for reactors
        self.createGases()
        #Optionally building network from initialization
        if self.build:
            self.buildNetwork()

    def __call__(self,time,fname=None):
        """Use this function to call the object and iterate the reactor through time."""
        self.network.advance(time)
        self.state = self.network.get_state()
        self.writer(self.state) #Write if file path is provided
        return self.state

    def __str__(self):
        """This overloads the print function."""
        if self.ptype:
            return output.sparsePrint(self)
        else:
            return output.densePrint(self)

    def steadyState(self):
        self.network.advance_to_steady_state()
        self.state = self.network.get_state()
        self.writer(self.state) #Write if file path is provided
        return self.state

    def buildNetwork(self):
        """Call this function to build the network."""
        self.createReactors() #This function creates reactors
        self.connectReactors()  #This function connects reactors with Adj matrxi
        self.network = ct.ReactorNet(self.reactors) #Create reactor network
        self.network.set_initial_time(0) #Set initial reactor time to zero
        self.network.reinitialize() #Initialize reactor network
        self.initialState = self.network.get_state()
        self.keys = [self.network.component_name(i) for i in range(len(self.initialState))]
        #Added h5writer if a file path is specified
        self.writer = (lambda x:0) if self.fpath is None else output.h5Writer(self.fpath,self.keys,self.initialState)

    def createGases(self):
        """This function creates gases to be used in building the reactor network.
        """
        #Building objects for fuel and air
        self.fuel = ct.Solution(self.mechs[0])
        self.atmosphere = ct.Solution(self.mechs[1])
        self.exhausts = [ct.Solution(mech) for mech in self.mechs[2:]]

    def createReactors(self):
        """Use this function to create exhaust stream network"""
        #Creating fuel reservoir
        self.fuel.equilibrate('HP')
        self.fuelReservoir = ct.Reservoir(contents=self.fuel,name='fuel')
        #Creating combustor
        self.reactors = ct.ConstPressureReactor(contents=self.fuel,name='combustor',energy='on'),
        #Creating exhaust reactors
        exCycle = it.cycle(self.exhausts)
        self.reactors += tuple([ct.ConstPressureReactor(contents=next(exCycle),name='exhaust{}'.format(i),energy='on') for i in range(self.nex)])
        #Creating atomspheric reactor
        self.atmosReservoir = ct.Reservoir(contents=self.atmosphere,name='atmosphere')
        self.exhaustReservoir = ct.Reservoir(contents=self.atmosphere,name='exhaust')

    def connectReactors(self):
        """Use this function to connect exhaust reactors."""
        #Connecting fuel res -> combustor
        def inflow(t): # inflow is a variable mass flow rate base on residence time
            return self.reactors[0].mass / self.residenceTime(t)
        self.inflow = inflow
        self.controllers = ct.MassFlowController(self.fuelReservoir,self.reactors[0],mdot=self.inflow), #Fuel Air Mixture MFC
        #Connecting combustor -> exhaust
        self.controllers += ct.MassFlowController(self.reactors[0],self.reactors[1],mdot=self.inflow), #Exhaust MFC
        #Connecting exhausts -> adj exhaust
        exStart = 1 #starting index of exhaust reactors
        for f, row in enumerate(self.connects[:-1],exStart):
            sink = np.sum(row) #number of places the mass flow goes from one reactor to another
            if sink != 0: #Connects non-terminal exhaust reactors
                for t,value in enumerate(row[:-1],start=exStart):
                    if value:
                        #Using a closure to create mass flow function
                        def mdot(t,fcn=None):
                            return (self.reactors[fcn.ridx].mass / self.residenceTime(t)) / fcn.sink
                        mdot.__defaults__ = (mdot,)
                        mdot.sink = sink #number of places the mass flow goes from one reactor to another
                        mdot.ridx = np.copy(f)
                        #Create controller
                        self.controllers += ct.MassFlowController(self.reactors[f],self.reactors[t],mdot=mdot), #Exhaust MFCS
            else: #This else statement connects all terminal exhaust reactors to exhaust reservoir
                #Using a closure to create mass flow function
                def mdot(t,fcn=None):
                    return (self.reactors[fcn.ridx].mass / self.residenceTime(t))
                mdot.__defaults__ = (mdot,)
                mdot.sink = sink #number of places the mass flow goes from one reactor to another
                mdot.ridx = np.copy(f)
                #Create controller
                self.controllers += ct.MassFlowController(self.reactors[f],self.exhaustReservoir,mdot=mdot), #Exhaust MFCS

        #entrainment controllers
        for t, value in enumerate(self.connects[-1],exStart):
            if value:
                self.controllers += ct.MassFlowController(self.atmosReservoir,self.reactors[t],mdot=self.entrainment), #Exhaust MFCS

    #Class methods that implement certain kinds of reactor ideas.
    @classmethod
    def simpleModel(cls,mechs=["gri30.cti","air.cti","gri30.cti"],residenceTime=lambda t: 0.1, entrainment=lambda t:0.1,fpath="simple.hdf5",setCanteraPath=None,build=False):
        """This classmethod build a 1 reactor exhaust model. It has extra parameters than the class
        Linear expansion model:
        [fuel res]->[combustor]->[ex1]->[exRes]
        [farfield]->[ex1]
        """
        n = 1
        connects = np.zeros((n+1,n+1))
        connects[1,0] = 1
        return cls(mechs,connects,residenceTime=residenceTime,entrainment=entrainment,fpath=fpath,setCanteraPath=setCanteraPath,build=build)

    @classmethod
    def linearExpansionModel(cls,n=10,mechs=["gri30.cti","air.cti","gri30.cti"],residenceTime=lambda t: 0.1, entrainment=lambda t:0.1,fpath="linear.hdf5",setCanteraPath=None,build=False):
        """ Use this function to generate an instance with linear expansion connects method. It takes all the parameters
            that the class does except connects and replaces connects with n parameter.

        Parameters:
            n - number of reactors using linear expansion. e.g. at level 1 there is one reactor
                at level two there are two and so on. n must result in an integer number of steps
                based on the formula:steps=(-1+np.sqrt(1+8*n))/2

        Linear expansion model:
        [fuel res]->[combustor]->[ex1]->[ex2]->[ex4]->[exRes]
                                      ->[ex3]->[ex5]->[exRes]
                                             ->[ex6]->[exRes]
        [farfield]->[ex1,ex2,ex3,ex4,ex6]

        Notes:
        The farfield is connected as an inlet for each exterior reactor if you were to draw them as 2D blocks.
        """
        connects = np.zeros((n+1,n+1))
        getsteps = lambda k: (-1+np.sqrt(1+8*k))/2
        x = 1
        j = 1
        i = 0
        steps = getsteps(n)
        #Check if is integer
        try:
            if steps.is_integer():
                steps = int(steps)
            else:
                raise TypeError("{}: is not an Integer as required by: steps=(-1+np.sqrt(1+8*n))/2".format(steps))
        except TypeError as e:
            tb = traceback.format_exc()
            print(tb)
            sys.exit()
            pass
        #Fill exhaust dependency
        for cn in range(2,2+steps-1):
            connects[i:i+x,j:j+x+1] = 1
            i+=x
            x = x+1
            j+=x
        #Connecting farfield
        idx = it.cycle(np.arange(0,n,1))
        for i in range(steps):
            if i+1 >= 3:
                connects[-1,next(idx)]=1
                for j in range(1,i):
                    connects[-1,next(idx)]=0
                connects[-1,next(idx)]=1
            else:
                for j in range(i+1):
                    connects[-1,next(idx)]=1
        return cls(mechs,connects,residenceTime=residenceTime,entrainment=entrainment,fpath=fpath,setCanteraPath=setCanteraPath,build=build)

    @classmethod
    def gridModel(cls,n=3,m=3,mechs=["gri30.cti","air.cti","gri30.cti"],residenceTime=lambda t: 0.1, entrainment=lambda t:0.1,fpath="grid.hdf5",setCanteraPath=None,build=False):
        """ Use this function to generate an instance with grid connects method. It takes all the parameters
            that the class does except connects and replaces connects with n parameter.

        Parameters:
            n - Integer number of reactor rows
            m - Integer number of reactor columns

        Grid model (3x3):
        [fuel res]->[combustor]->[ex0]->[ex1]->[ex4]->[ex7]->[exRes]
                                      ->[ex2]->[ex5]->[ex8]->[exRes]
                                      ->[ex3]->[ex6]->[ex9]->[exRes]

        [farfield]->[ex0,ex1,ex7,ex4,ex3,ex6,ex9]

        Notes:
        The farfield is connected as an inlet for each exterior reactor if you were to draw them as 2D blocks.
        """
        clen = int(n*m+2)
        connects = np.zeros((clen,clen))
        #Fill reactor 0
        for i in range(n):
            connects[0,i+1] = 1
        #Fill remaining
        for i in range(1,clen-n-1):
            connects[i,i+n] = 1
        #Connect farfield
        connects[-1,0] = 1
        for i in range(1,clen-n,n):
            connects[-1,i]=1
            connects[-1,i+n-1]=1
        return cls(mechs,connects,residenceTime=residenceTime,entrainment=entrainment,fpath=fpath,setCanteraPath=setCanteraPath,build=build)

def modelCLI():
    """This function is used for the command line interface option of mech"""
    parser = argparse.ArgumentParser(description="""This is the commandline interface for
    running an exhuast network.
    """)

    fmap = {'simple' : PlumeModel.simpleModel,
                    'grid' : PlumeModel.gridModel,
                    "linear":PlumeModel.linearExpansionModel}
    parser.add_argument('network', choices=fmap.keys(),help="This is a required arguement that specifies the model which will be used. Currently implemented choices are simple, grid, and linear.")
    parser.add_argument("-ss","--steady",action='store_true',help="""set this flag run to steady state after integration""")
    parser.add_argument("-t0",nargs="?",default=0,type=float,help="Initial integration time")
    parser.add_argument("-tf",nargs="?",default=1,type=float,help="Final integration time")
    parser.add_argument("-dt",nargs="?",default=0.1,type=float,help="Integration time interval")
    parser.add_argument("-t","--test",action='store_true',help="""set this flag to run test functions.""")
    parser.add_argument("-v","--verbose",action='store_true',help="""set this flag to run print statements during the process.""")

    args = parser.parse_args()
    if args.verbose:
        print("Creating {} model and building network.".format(args.network))
    pm = fmap[args.network]()
    pm.buildNetwork()

    for t in np.arange(args.t0,args.tf+args.dt,args.dt):
        if args.verbose:
            print("Advancing to time: {:0.3f}.".format(t))
        pm(t)

    if args.steady:
        if args.verbose:
            print("Advancing to steady state.")
        pm.steadyState()

    if args.test:
        if args.verbose:
            print("Running model test suite.")
        pyplume.tests.testModel.runTests()

if __name__ == "__main__":
    # modelCLI()
    pm = PlumeModel.simpleModel()
    pm.buildNetwork()
    for t in np.arange(0,1,0.1):
        pm(t)
    



import setuptools

with open("README.md", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyplume",
    version="0.0.1",
    author="Anthony Walker",
    author_email="walkanth@oregonstate.edu",
    license='BSD 3-clause "New" or "Revised License"',
    description="This package contains files to build models for exhaust plume analysis and methods to analyze the results.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SoftwareDevEngResearch/plume-generation-and-analysis",
    entry_points={
        'console_scripts': [
            'pyplume.mech=pyplume.mech:mechCLI',
            'pyplume.model=pyplume.model:modelCLI'
        ]
    },
    packages=setuptools.find_packages(),
    package_dir={'pyplume':'./pyplume', 'tests':'./pyplume/tests'},
    package_data={'pyplume': ['mechanisms/*','originals/*','tests/*']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['pytest','cantera','h5py']
)



import h5py
import numpy as np

def statementTVM(pReact):
    """Use this funciton to produce the TVM statemet"""
    T,V,mass = pReact.T,pReact.volume,pReact.mass
    statement="\n{}: T: {:0.2f} K, V: {:0.2f} m^3, mass: {:0.2f} kg".format(pReact.name,T,V,mass)
    return statement
#Printing methods
def sparsePrint(plume):
    """This overloads the print function."""
    statement = 'PyPlume Network Model Summary:'
    statement += statementTVM(plume.fuelReservoir)
    statement += statementTVM(plume.atmosReservoir)
    for react in plume.reactors:
        statement += statementTVM(react)
    statement += statementTVM(plume.exhaustReservoir)
    return statement

def densePrint(plume):
    """This overloads the print function."""
    statement = 'PyPlume Network Model Summary:'
    statement += statementTVM(plume.fuelReservoir)
    statement += statementTVM(plume.atmosReservoir)

    for react in plume.reactors:
        statement += statementTVM(react)
    statement += statementTVM(plume.exhaustReservoir)
    statement += '\nReactor Network Mass Fractions:'
    keys = []
    added = []
    state=plume.network.get_state()

    for i in range(len(state)):
        name = plume.network.component_name(i)
        keys.append(name)

    for i,key in enumerate(keys):
        keyname,element = key.split(":")
        if keyname not in added:
            added.append(keyname)
            statement+="\n{}\n".format(keyname)
        statement+="{}: {:0.2e}, ".format(element,state[i])
    return statement
#External methods to writer class

def stringToOrd(strList):
    """Use this function to convert a string to an ord value."""
    keyOrds = []
    for cstr in strList:
        for ch in cstr:
            keyOrds.append(ord(ch))
        keyOrds.append(ord(','))
    return keyOrds

def ordToString(ordList):
    """Use this function to convert ord values to strings."""
    newStrList = []
    cstr = ""
    for cint in ordList:
        cstr += chr(cint)
        if cint == 44:
            newStrList.append(cstr[:-1])
            cstr = ""
    return newStrList

#Writer class
class h5Writer(object):
    """docstring for h5Writer."""

    def __init__(self, fpath, keys, initial, chunk=100):
        super(h5Writer, self).__init__()
        self.fpath = fpath
        self.keys = keys
        self.f = h5py.File(fpath,'w')
        self.chunk = chunk
        self.dshape = [self.chunk,np.shape(initial)[0]]
        self.elements = {key.split(":")[1][1:]:i for i,key in enumerate(self.keys)}
        self.keys = [key.split(":")[0] for key in self.keys]
        self.createSlices()
        self.createDataSets()
        self.storeData()

    def __call__(self,data):
        """Use the call function to write data to current hdf5 handle."""
        try:
            #Try to add to file
            for key in self.ukeys:
                self.f[key][self.time[key],self.slices[key]] = data[self.slices[key]]
        except Exception as e:
            print(str(e)+': resizing hdf5 dataset')
            #Adding now that is has been resized
            for key in self.ukeys:
                self.f[key].resize(self.dshape[0]+self.chunk,axis=0)
                self.f[key][self.time[key],self.slices[key]] = data[self.slices[key]]
            self.dshape[0]+=self.chunk
        finally:
            #Updating times
            for key in self.ukeys:
                self.time[key]+=1

    def __del__(self):
        """This is the h5Writer destructor."""
        # self.f.close() #Close file on destruction
        pass

    def createSlices(self):
        """Use this function to create index slices."""
        #Creating unique keys
        self.ukeys = []
        for key in self.keys:
            if key not in self.ukeys:
                self.ukeys.append(key)
        #Creating slices
        self.slices = {key:0 for key in self.ukeys}
        prev = 0
        for key in self.ukeys:
            ct = self.keys.count(key)
            self.slices[key] = slice(prev,prev+ct,1)
            prev = ct

    def createDataSets(self):
        """This functions creates data sets in the file."""
        self.time = {}
        for key in self.ukeys:
            self.f.create_dataset(key, self.dshape, dtype=np.float64,chunks=tuple(self.dshape),maxshape=(None,None))
            self.time[key]=0 #Store current time

    def storeData(self):
        """Use this function to store data used to manage the hdf5 file."""
        #Adding keys
        tkeys=stringToOrd(self.keys)
        self.f.create_dataset('keys', np.shape(tkeys), dtype=int,data=tkeys)
        # print(self.elements)
        #Adding ukeys
        tkeys=stringToOrd(self.ukeys)
        self.f.create_dataset('ukeys', np.shape(tkeys), dtype=int,data=tkeys)
        #Adding slices
        slNums = []
        for key in self.ukeys:
            sl = self.slices[key]
            slNums.append(sl.start)
            slNums.append(sl.stop)
        self.f.create_dataset('slices', np.shape(slNums), dtype=int, data=slNums)
        self.f.create_dataset('chunk' ,(1,) ,dtype=int, data=self.chunk)
        ttime = [ self.time[key] for key in self.time]
        self.f.create_dataset('time',(len(self.time),),dtype=int,data=ttime)
        self.f.create_dataset('dshape',(len(self.dshape),),dtype=int,data=self.dshape)
        # print("Add keys here")

    def setVars(self,time,chunk,slices,dshape):
        """This function is used by existingFile to set varibales for consistency."""
        self.time = time
        self.chunk = chunk
        self.slices = slices
        self.dshape = dshape

    @classmethod
    def existingFile(cls,fpath):
        """Use this to generate a writer for an existing file.
        Parameters:
        fpath - path to existing hdf5 file.
        """
        f = h5py.File(fpath,'r+')
        args = fpath,ordToString(list(f['keys'])),np.zeros(f['dshape'])
        vars = f['time'],f['chunk'],f['slices'],f['dshape']
        f.close()
        h5w = cls(*args)
        h5w.setVars(*vars)
        return h5w

if __name__ == "__main__":
    h5Writer.existingFile("simple.hdf5")
